/**
 * PAUL HUSSMAN
 * 260224172
 * COMP 409 ASSIGNMENT 4
 * FALL 2010
 */


import java.util.LinkedList;
import java.awt.*;
import java.awt.geom.*;

/** A Channel is used to connect two Functions.
 * It provides a send and receive method.  
 * Both are blocking if there is not enough capacity
 * or there is no token present on the Channel. */
public class Channel<T> extends Function {

    /** Maximum capacity of channels. */
    public static final int CAPMAX = 20;
    
    public static int capmin = 0;

    /** Destination coords. */
    public int dx,dy;

    /** Font used for token labels. */
    public static Font tFont = new Font("SansSerif", Font.PLAIN, ((int)(10*Circuit.multiplier)));
    
    /** Capacity of the Channel. */
    private static int capacity = 1;

    /** Ordered list of tokens on the Channel . */
    private LinkedList<T> tokens = new LinkedList<T>();

    /** Input Channel if Channels are chained together. */
    private Channel<T> fromChannel;

    /** Token radius. */
    private static final int TR = ((int)(8*Circuit.multiplier));

    /** Arrow height. */
    private static final int AH = ((int)(6*Circuit.multiplier));

    /** Arrow width. */
    private static final int AW = ((int)(2*Circuit.multiplier));

    /** For tokens, colour to draw a boolean true. */
    private static Color trueC = new Color(0xff<<24|54<<16|225<<8|26);

    /** For tokens, colour to draw a boolean false. */
    private static Color falseC = new Color(0xff<<24|248<<16|93<<8|25);

    /** For tokens, colour to draw an integer. */
    private static Color intC = new Color(0xff<<24|225<<16|204<<8|26);

    /** Constructor. */
    public Channel() {
        c = Color.black;
    }

    /** An active channel has an associated thread. */
    public boolean isActive() {
        return (fromChannel!=null);
    }
    
	Object block = new Object();
	private boolean isEmpty = true;
	private boolean isFull = false;
	private boolean lookingAtList;
    

    /** Draw the Channel.  Also draws the tokens. */
    public synchronized void draw(Graphics gr) {
        gr.setColor(c);
        gr.drawLine(x,y,dx,dy);
        if (tokens.size()>0) {
            // space out tokens evenly along the line
            int changex;
            int changey;
            if (dx-x!=0) {
                // horizontal line
                int len = dx-x;
                changex = len/(tokens.size()+1);
                changey = 0;
            } else {
                // vertical line
                int len = dy-y;
                changey = len/(tokens.size()+1);
                changex = 0;
            }
            // draw tokens
            int i=1;
			   lookingAtList = true;
               for (T token : tokens) {
                   int tx,ty;
                   tx = x+changex*i;
                   ty = y+changey*i;
                   drawToken(gr,token,tx,ty);
                   i++;

               }
               lookingAtList = false;
        }
        // and draw arrows
        drawArrows(gr);
    }

    /** Draw a single token. */
    public static void drawToken(Graphics gr,Object token,int tx,int ty) {
        String s="?";
        gr.setColor(Color.darkGray);
        gr.fillOval(tx-TR-2,ty-TR+2,TR*2,TR*2);
        if (token instanceof Boolean) {
            Boolean b = (Boolean)token;
            if (b.booleanValue()==false) {
                gr.setColor(falseC);
                s = "F";
            } else {
                gr.setColor(trueC);
                s = "T";
            }
            gr.fillOval(tx-TR,ty-TR,TR*2,TR*2);
            gr.setColor(Color.black);
        } else if (token instanceof Integer) {
            s = ((Integer)token).toString();
            gr.setColor(intC);
            gr.fillOval(tx-TR,ty-TR,TR*2,TR*2);
            gr.setColor(Color.black);
        }
        gr.setFont(tFont);
        FontMetrics fm = gr.getFontMetrics();
        Rectangle2D r = fm.getStringBounds(s,gr);
        gr.drawString(s,tx-(int)(r.getWidth()/2),ty+(int)(r.getHeight()/2)-1);
    }

    /** Draw the arrows on the Channel. */
    protected void drawArrows(Graphics g) {
        g.setColor(Color.black);
        if(dx-x==0) {
            if (dy-y>0) {
                g.drawLine(dx-AW,dy-AH,dx,dy);
                g.drawLine(dx+AW,dy-AH,dx,dy);
            } else {
                g.drawLine(dx-AW,dy+AH,dx,dy);
                g.drawLine(dx+AW,dy+AH,dx,dy);
            }
        } else {
            if (dx-x>0) {
                g.drawLine(dx-AH,dy-AW,dx,dy);
                g.drawLine(dx-AH,dy+AW,dx,dy);
            } else {
                g.drawLine(dx+AH,dy-AW,dx,dy);
                g.drawLine(dx+AH,dy+AW,dx,dy);
            }
        }
    }
    
    /** Link the output of this Channel to the input of another. */
    // only used during initial setup
    public void linkTo(Channel<T> toC) {
        toC.fromChannel = this;
    }

    /** Link the input of this Channel to the output of another. */
    // only used during initial setup
    public void linkFrom(Channel<T> fromC) {
        fromChannel = fromC;
    }

    /** Only active Channels, ones connected as part of a chain 
     * of Channels will fire, the rest rely on their attached 
     * functions to keep things going. */
    protected void canFire() {
        fromChannel.mustReceive();
        mustSend();
    }

    protected void fire() {
        T i = fromChannel.receive();
        send(i);
    }

    /** Assumes Channels have been linked already. */
    public void setPoints() {
        if (fromChannel!=null) {
            x = fromChannel.dx;
            y = fromChannel.dy;
        }
    }
    
    /** Return whether there is capacity in this Channel. */
    public synchronized boolean full() {
        return tokens.size()>=getCapacity();
    }
    
    /** Returns the capacity. */
    public synchronized static int getCapacity() {
        return capacity;
    }
    
    
    /** Tries to increase the capacity of this Channel. */
    public synchronized static void increaseCapacity() {
        if (capacity<CAPMAX) {
            capacity++;
        }
    }
    
    
    /** Tries to decrease the capacity of this Channel. */
    public synchronized static void decreaseCapacity() {
		if (capacity>1){
            capacity--;
		}
    }
    
    
    /** Let this Channel know that capacity increased. */
    public synchronized void capacityIncreased() {
    	isFull = false;
    	notifyAll();
    }

	public void capacityDecreased() {
    	//
	}
    
    /** Check out the next token that would be received. */
    public synchronized T peek() {
        return tokens.getLast();
    }


    /** Block until something can be received from this Channel. */
    public synchronized void mustReceive() {

        while (isEmpty) {
        	
        	try {
        		this.wait();
        	} catch (InterruptedException e) {
        		
        	}
        }
    }

    /** Receive something from this Channel. */
    public synchronized T receive() {

        while (lookingAtList); 
        
        T token = tokens.removeLast();
        
        if (isFull) {
        		this.notify();
        		isFull = false;
        }

        if (tokens.size()==0) {
        	isEmpty = true;
        }

        return token;
    }

    /** Block until sending something into this Channel is possible. */
    public synchronized void mustSend() {

        while (isFull) {
        		try {
        			this.wait();
        		} catch (InterruptedException e) {}
        }
    }

    
    /** Send something into this Channel. */
    public synchronized void send(T token) {

        while (lookingAtList); 
        
        tokens.addFirst(token);
        
        if (isEmpty) {
        		notifyAll();
        		isEmpty = false;
        }
        if (full()) {
        	isFull = true;
        }
    }

    
    public void setFalse() {
        if (tokens.size()!=0){
        	isEmpty = false;
        }
        if (full()){
        	isFull = true;
        }
    }

}