/**
 * PAUL HUSSMAN
 * 260224172
 * COMP 409 ASSIGNMENT 4
 * FALL 2010
 */

import java.util.ArrayList;
import java.awt.*;

/** Main class composing the circuit.  Responsible for creating
 * all circuit components, starting, stopping etc,  and drawing the 
 * simulation. */
public abstract class Circuit {

    /** For debug messages. */
    public static final boolean debug = false;

    /** A drawing scaling multipler. 1.0 is best. */
    public static final double multiplier = 1.0;

    /** The size of one unit in pixels. */
    public static final int SCALE = ((int)(20*multiplier));

    /** Width of the GUI in logical, unscaled units. */
    public static int GUIW = 42;

    /** Height of the GUI in logical, unscaled units. */
    public static int GUIH = 19;

    /** All functions registered. */
    protected ArrayList<Function> funcs = new ArrayList<Function>();

    /** Height of the speed bar.  Not scaled. */
    private static final int BARH = 12;

    /** Increment for speed. */
    private static final int DELAYINCR = 100;

    /** Maximum delay in ms. */
    private static final int DELAYMAX = 2000;

    /** Actual delay speed in ms. */
    private static int delaySpeed = 1000;

    /** A flag to indicate if threads were started. */
    private boolean started;

    /** Colour used for speed bar. */
    private static Color barC = new Color(0xff<<24|29<<16|179<<8|54);

    /** Colour used for capacity bar. */
    private static Color capC = new Color(0xff<<24|179<<16|29<<8|54);

	public static boolean running = true;

	public static volatile boolean paused = false;

    /** Draw the entire circuit. */
    public void draw(Graphics g) {
        // all drawing is delegated
        for (Function f : funcs) {
            f.draw(g);
        }
    }

    /** Draw the speed bar. */
    public void drawSpeed(Graphics g,int maxw,int maxh) {
        g.setColor(barC);
        g.fillRect(4,maxh-BARH-8,(int)((((double)(DELAYMAX-delaySpeed))/DELAYMAX)*(maxw-16)),BARH/2);
        g.setColor(capC);
        
        g.fillRect(4,maxh-BARH/2-8,(int)((((double)(Channel.getCapacity()))/
                                          Channel.CAPMAX)*(maxw-16)),BARH/2);
        g.setColor(Color.black);
        g.drawRect(4,maxh-BARH-8,maxw-16,BARH/2);
        int w = (maxw-16)/Channel.CAPMAX;
        for (int i=0;i<Channel.CAPMAX;i++) {
            g.drawRect(4+i*w,maxh-BARH/2-8,w,BARH/2);
        }
    }

    /** Construct the circuit.  Allocates all components and
     * assigns coordinates. Does not start any execution. All
     * subclasses must implement this to construct their appropriate
     * circuit. */
    public abstract void build();

    ArrayList<Thread> threads = new ArrayList<Thread>();
    
    /** Start the simulation. Launches threads for all active components. */
    public void start() {
        if (!started) {
        	
            for (Function f : funcs) {
                if (f.isActive()){
                    threads.add(new Thread(f));
                	threads.get(threads.size()-1).start();
                }
            }
            started = true;
            running = true;
            
        }
    }

    /** Return the delay. */
    public static int getDelay() {
        return delaySpeed;
    }

    /** Increase channel capacity. */
    public void moreCapacity() {
        // increase the flag
        Channel.increaseCapacity();
        // and make sure everyone knows
        for (Function f : funcs) {
            if (f instanceof Channel) {
                Channel c = (Channel)f;
                c.capacityIncreased();
            }
        }
    }
    
	public void lessCapacity() {
        // decrease the flag
        Channel.decreaseCapacity();
        // and make sure everyone knows
        for (Function f : funcs) {
            if (f instanceof Channel) {
                Channel c = (Channel)f;
                c.capacityDecreased();
            }
        }
	}

    /** Speed up the simulation. */
    public void faster() {
        if (delaySpeed>=DELAYINCR)
            delaySpeed-=DELAYINCR;
    }

    /** Slow down the simulation. */
    public void slower() {
        if (delaySpeed+DELAYINCR<=DELAYMAX)
            delaySpeed+=DELAYINCR;
    }

    /** Pause the simulation. */
    public void pause() {
    	paused = true;
    }

    /** Resume the simulation. */
    public void resume() {
    	paused = false;
    	for (Thread t : threads){
    		t.interrupt();
    	}
    	
    }

    /** Stop the simulation. */
    public void stop() {
    	running = false;
    }



}
