import java.util.ArrayList;
import java.awt.*;

/** A Circuit which emits squares of the non-negatives. */
public class CircuitSquares extends Circuit {

    /** Set the size in the constructor. */
    public CircuitSquares() {
        GUIW = 42;
        GUIH = 19;
    }

    /** It's really composed of three pieces
     *    1) a loop to generate all non-negatives
     *    2) compute partial sums (n^2+n)/2
     *    3) sum pairs  (n^2+n)/2+((n+1)^2+n+1)/2
     **/
    public void build() {
        int i;
        // allocate channels
        ArrayList<Channel<Integer>> ic = new ArrayList<Channel<Integer>>();
        for (i=0;i<16;i++) 
            ic.add(new Channel<Integer>());

        // connect channels to make longer paths
        ic.get(1).linkTo(ic.get(2));
        ic.get(2).linkTo(ic.get(3));

        ic.get(4).linkTo(ic.get(0));

        ic.get(7).linkTo(ic.get(8));
        ic.get(8).linkTo(ic.get(9));
        ic.get(9).linkTo(ic.get(10));

        // add any branching
        FunctionFork<Integer> fork1 = new FunctionFork<Integer>(ic.get(3),ic.get(4),ic.get(5));
        FunctionFork<Integer> fork2 = new FunctionFork<Integer>(ic.get(6),ic.get(7),ic.get(11));
        FunctionFork<Integer> fork3 = new FunctionFork<Integer>(ic.get(11),ic.get(12),ic.get(13));
        
        // now create functions
        FunctionPlus plus1 = new FunctionPlus(ic.get(5),ic.get(10),ic.get(6));
        FunctionPlus plus2 = new FunctionPlus(ic.get(13),ic.get(14),ic.get(15));

        FunctionInc inc = new FunctionInc(ic.get(0),ic.get(1),true);

        FunctionCDR<Integer> cdr = new FunctionCDR<Integer>(ic.get(12),ic.get(14));

        FunctionOutput output = new FunctionOutput(ic.get(15),true);

        // assign coordinates to components

        plus1.x = 13*SCALE; plus1.y = 2*SCALE; plus1.w = 2*SCALE; plus1.h = 8*SCALE;
        plus2.x = 27*SCALE; plus2.y = 4*SCALE; plus2.w = 2*SCALE; plus2.h = 8*SCALE;

        inc.x = 2*SCALE; inc.y = 7*SCALE; inc.w = 4*SCALE; inc.h = 2*SCALE;

        cdr.x = 20*SCALE; cdr.y = 9*SCALE; cdr.w = 4*SCALE; cdr.h = 2*SCALE;

        output.x = 32*SCALE; output.y = 7*SCALE; output.w = 8*SCALE; output.h = 2*SCALE;

        fork1.x = 8*SCALE; fork1.y = 4*SCALE;
        fork2.x = 18*SCALE; fork2.y = 6*SCALE;
        fork3.x = 22*SCALE; fork3.y = 6*SCALE;

        plus1.setPoints(); plus2.setPoints();
        inc.setPoints();
        cdr.setPoints();
        output.setPoints();
        fork1.setPoints(); fork2.setPoints(); fork3.setPoints(); 

        // assign coordinates to chains of channels.  Just need to
        // assign hanging destinations.  All channels must be axis-aligned.
        ic.get(1).dx = 4*SCALE; ic.get(1).dy = 12*SCALE;
        ic.get(2).dx = 8*SCALE; ic.get(2).dy = 12*SCALE;

        ic.get(4).dx = 4*SCALE; ic.get(4).dy = 4*SCALE;

        ic.get(7).dx = 18*SCALE; ic.get(7).dy = 12*SCALE;
        ic.get(8).dx = 10*SCALE; ic.get(8).dy = 12*SCALE;
        ic.get(9).dx = 10*SCALE; ic.get(9).dy = 8*SCALE;

        // set remaining start points for channels and
        // add all components to the funcs list
        for (Channel<Integer> c : ic) {
            c.setPoints();
            funcs.add(c);
        }
        funcs.add(fork1);
        funcs.add(fork2);
        funcs.add(fork3);
        funcs.add(plus1);
        funcs.add(plus2);
        funcs.add(inc);
        funcs.add(cdr);
        funcs.add(output);

        // create initial tokens
        ic.get(0).send(new Integer(0));
        ic.get(6).send(new Integer(0));
    }
}
