/**
 * PAUL HUSSMAN
 * 260224172
 * COMP 409 ASSIGNMENT 4
 * FALL 2010
 */

import java.awt.*;
import java.awt.geom.*;

/** Main abstract superclass for functions.  These can be
 * executed in Threads (ie are Runnable). */
public abstract class Function implements Runnable {

    /** Reference coordinates. */
    public int x,y;

    /** Dimensions. */
    public int w,h;

    /** Colour of the component.  */
    public Color c = new Color(0xff<<24|14<<16|169<<8|208);

    /** Font used for labels. */
    public static Font f = new Font("SansSerif", Font.BOLD, ((int)(12*Circuit.multiplier)));

    /** Actual label. */
    public String label;

    /** Size of a rounded corner radius. */
    private static final int RR = ((int)(24*Circuit.multiplier));

    /** An active channel has an associated thread. */
    public boolean isActive() {
        return true;
    }

    /** Default drawing for components. Draw a rounded rectangle,
     * then center the label inside. */
    public void draw(Graphics g) {
        g.setColor(Color.darkGray);
        g.fillRoundRect(x,y,w,h,RR,RR);
        g.setColor(c);
        g.fillRoundRect(x,y,w,h,RR,RR);
        g.setColor(Color.black);
        g.drawRoundRect(x,y,w,h,RR,RR);
        if (label!=null) {
            g.setColor(Color.black);
            g.setFont(f);
            FontMetrics fm = g.getFontMetrics();
            Rectangle2D r = fm.getStringBounds(label,g);
            g.drawString(label,x+w/2-(int)(r.getWidth()/2),y+h/2+(int)(r.getHeight()/2));
        }
    }

    /** Template run method.  Check whether it can fire, and 
     * if so, does.  Sleeps a bit in between.  */
    public void run() {
        while(Circuit.running) {
        	
        	if(Circuit.paused){
        		pauseCircuit();
        	}
        	
    		canFire();
        	
    		delay();
    		
        	if(Circuit.paused){
        		pauseCircuit();
        	}
    		fire();	
        }
    }

    /** Core functionality; detect whether this component can fire.
     * A component can fire if all input is available and any
     * output channels have space. This should block until firing is
     * certainly possible. */
    protected abstract void canFire();

    /** Routine to actually execute its action, reading input
     * and emitting output.  Assumes firing is possible. */
    protected abstract void fire();
    
    /** Assuming the coordinates and size of this component have been
     * set, set the coordinates of all attached Channels. */
    public abstract void setPoints();

    /** Delay before firing.  */
    protected static void delay() {
        try {
            Thread.sleep(Circuit.getDelay());
        } catch(InterruptedException ie) {
        }
    }
    
    // pauses the circuit
    public void pauseCircuit() {
	    while(Circuit.paused) {
			try {
				Thread.sleep(100000);
			} catch (InterruptedException e) {
				
			}
	    }
	}
}
