import java.awt.*;
import java.awt.geom.*;

/** A non-regular actor, merge uses a boolean control
 * line to switch which input line it reads from. */
public class FunctionMerge<T> extends Function {
    /** Channels. */
    private Channel<T> out,in1,in2;
    private Channel<Boolean> ctrl;

    /** A flag to indicate ports are drawn reflected horizontally. */
    private boolean refl;

    public FunctionMerge(Channel<T> in1,Channel<T> in2,Channel<T> out,Channel<Boolean> ctrl) {
        this.in1 = in1;
        this.in2 = in2;
        this.out = out;
        this.ctrl = ctrl;
        label = "Merge";
    }

    /** For specifing it's reflected. */
    public FunctionMerge(Channel<T> in1,Channel<T> in2,Channel<T> out,Channel<Boolean> ctrl,boolean refl) {
        this(in1,in2,out,ctrl);
        this.refl = refl;
    }

    public void setPoints() {
        if (refl)
            in1.dx = x+w/2+w/4;
        else
            in1.dx = x+w/4;
        in1.dy = y;
        if (refl) 
            in2.dx = x+w/4;
        else
            in2.dx = x+w/2+w/4;
        in2.dy = y;
        out.x = x+w/2;
        out.y = y+h;
        ctrl.dy = y+h/2;
        if (refl)
            ctrl.dx = x;
        else
            ctrl.dx = x+w;
    }

    /** Slight complexity; need to only check the inputs we would
     * certainly read. */
    protected void canFire() {
        out.mustSend();
        ctrl.mustReceive();
        Boolean b = ctrl.peek();
        if ((b.booleanValue()==true))
            in1.mustReceive();
        else
            in2.mustReceive();
    }

    protected void fire() {
        Boolean b = ctrl.receive();
        T i;
        if (b.booleanValue()) {
            i = in1.receive();
        } else {
            i = in2.receive();
        }
        if (Circuit.debug)
            System.out.println(label+": "+i);
        out.send(i);
    }

    /** Override draw so port labels can be drawn. */
    public void draw(Graphics g) {
        super.draw(g);
        drawPortLabels(g);
    }

    /** For drawing port labels. */
    protected void drawPortLabels(Graphics g) {
        g.setColor(Color.black);
        g.setFont(Channel.tFont);
        FontMetrics fm = g.getFontMetrics();
        Rectangle2D r = fm.getStringBounds("T",g);
        if (refl) {
            g.drawString("F",x+w/4-(int)(r.getWidth())-((int)(3*Circuit.multiplier)),
                         y-((int)(2*Circuit.multiplier)));
            g.drawString("T",x+w/2+w/4+((int)(4*Circuit.multiplier)),
                         y-((int)(2*Circuit.multiplier)));
        } else {
            g.drawString("T",x+w/4-(int)(r.getWidth())-((int)(3*Circuit.multiplier)),
                         y-((int)(2*Circuit.multiplier)));
            g.drawString("F",x+w/2+w/4+((int)(4*Circuit.multiplier)),
                         y-((int)(2*Circuit.multiplier)));
        }

    }
}
