import java.awt.*;
import java.awt.geom.*;

/** A non-regular actor, switch uses a boolean control
 * line to switch which input line it writes to. */
public class FunctionSwitch<T> extends Function {
    /** Channels. */
    private Channel<T> in,out1,out2;
    private Channel<Boolean> ctrl;

    /** A flag to indicate ports are drawn reflected horizontally. */
    private boolean refl;

    public FunctionSwitch(Channel<T> in,Channel<T> out1,Channel<T> out2,Channel<Boolean> ctrl) {
        this.in = in;
        this.out1 = out1;
        this.out2 = out2;
        this.ctrl = ctrl;
        label = "Switch";
    }

    /** For specifing it's reflected. */
    public FunctionSwitch(Channel<T> in,Channel<T> out1,Channel<T> out2,Channel<Boolean> ctrl,boolean refl) {
        this(in,out1,out2,ctrl);
        this.refl = refl;
    }

    public void setPoints() {
        in.dx = x+w/2;
        in.dy = y;
        if (refl)
            out1.x = x+w/2+w/4;
        else
            out1.x = x+w/4;
        out1.y = y+h;
        if (refl)
            out2.x = x+w/4;
        else
            out2.x = x+w/2+w/4;
        out2.y = y+h;
        ctrl.dy = y+h/2;
        if (refl)
            ctrl.dx = x;
        else
            ctrl.dx = x+w;
    }

    /** Slight complexity; need to only check the outputs we would
     * certainly write. */
    protected void canFire() {
        in.mustReceive();
        ctrl.mustReceive();
        Boolean b = ctrl.peek();
        if (b.booleanValue()==true)
            out1.mustSend();
        else
            out2.mustSend();
    }

    protected void fire() {
        T i = in.receive();
        Boolean b = ctrl.receive();
        if (Circuit.debug)
            System.out.println(label+": "+i);
        if (b.booleanValue()) 
            out1.send(i);
        else
            out2.send(i);
    }

    /** Override draw so port labels can be drawn. */
    public void draw(Graphics g) {
        super.draw(g);
        drawPortLabels(g);
    }

    /** For drawing port labels. */
    protected void drawPortLabels(Graphics g) {
        g.setColor(Color.black);
        g.setFont(Channel.tFont);
        FontMetrics fm = g.getFontMetrics();
        Rectangle2D r = fm.getStringBounds("T",g);
        if (refl) {
            g.drawString("F",x+w/4-(int)(r.getWidth())-2,y+h+(int)r.getHeight());
            g.drawString("T",x+w/2+w/4+3,y+h+(int)r.getHeight());
        } else {
            g.drawString("T",x+w/4-(int)(r.getWidth())-2,y+h+(int)r.getHeight());
            g.drawString("F",x+w/2+w/4+3,y+h+(int)r.getHeight());
        }

    }
}
