/**
 * PAUL HUSSMAN
 * 260224172
 * COMP 409 ASSIGNMENT 4
 * FALL 2010
 */

// Comp 409 Assignment 4 Framework
//
// Note that most Swing functions are *NOT* thread safe---you
// shouldn't be calling them concurrently!  If you only call
// Swing functions from your event handler thread you are generally
// safe.  See this link for more details:
//   http://java.sun.com/docs/books/tutorial/uiswing/mini/threads.html
// (that means of course any gui changes other threads want to make
// should be communicated through shared variables to the event
// handling thread).  Calling repaint is ok though.
//
// This code is in the public domain, and may be freely modified,
// copied, folded, stapled, or used in any fashion.

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/** The GUI will be an extension of a JFrame, a top-level window that
 * can include menus, a basic window-manager frame etc. */
public class GUI extends JFrame {

    /** The actual main function. Starts the GUI, which 
     * starts the simulation. */
    public static void main(String[] args) {
        int w,h;
        String cname = "CircuitLoop";

        // first arg if any is the circuit name
        if (args.length>0) {
            cname = "Circuit"+args[0];
        }

        // Load the circuit by reflection so we can pass the name
        // in as an argument
        try {
            Class<Circuit> cls = (Class<Circuit>)Class.forName(cname);
            Circuit c = cls.newInstance();
            GUI mainframe = new GUI(c);
        } catch(ClassNotFoundException cnfe) {
            System.out.println(cnfe);
        } catch(InstantiationException ie) {
            System.out.println(ie);
        } catch(IllegalAccessException iae) {
            System.out.println(iae);
        }
    }

    /** Construct the entire GUI. */
    public GUI(Circuit circuit) {
        super("Assignment 4  COMP-409  Winter 2008");

        // add a closing adaptor to respond to window close events
        // this is so you can click on the window-close button on the
        // main frame and it will go away.
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.out.println("Quitting main frame....");
                    System.exit(0);
                }
            });

        // Now, add some components to our GUI (without these it's just
        // a big empty frame)

        // the contents of a jframe are not accessed directly, rather
        // you go through the ContentPane, which is guaranteed to be non-null
        Container c = getContentPane();
        
        // in order to add components like buttons etc, we need a
        // layout manager.  One of the simplest ones is BorderLayout,
        // which conveniently enough is the default.  If not we'd
        // use code like:
        //c.setLayout(new BorderLayout());

        // inside we'll have two panels; one to contain buttons, the
        // other to contain the drawing area.  
        DrawingArea drawingArea = new DrawingArea(circuit);
        ButtonManager buttonList = new ButtonManager(drawingArea);

        // add these main components to the frame's content
        // the drawing area is centered (most of the window), abd
        // the button list will be in a row underneath it.
        c.add(drawingArea,BorderLayout.CENTER);
        c.add(buttonList,BorderLayout.SOUTH);

        // arrange components
        pack();
        // change size
        setSize(circuit.GUIW*Circuit.SCALE,
                circuit.GUIH*Circuit.SCALE);
        // set visible so window appears on screen
        setVisible(true);
    }
}


/** A manager for the list of buttons. */
class ButtonManager extends JPanel implements ActionListener {

    /** actual button objects. */
    protected JButton b0,b1,b2,b3,b4,b5,b6;
    protected JButton b7;	// decrease capacity

    /** drawing area we need to interact with. */
    protected DrawingArea d;

    /** Build a button manager communicating with a DrawingArea. */
    public ButtonManager(DrawingArea d) {
        // the button manager will hold a list of buttons. This requires
        // a flow layout
        setLayout(new FlowLayout());

        // store the drawing area for later use
        this.d = d;

        // Here's some buttons:
        b0 = new JButton("Start");
        b0.setActionCommand("start");
        b1 = new JButton("Pause");
        b1.setActionCommand("pause");
        b2 = new JButton("Continue");
        b2.setActionCommand("continue");
        b3 = new JButton("Faster");
        b3.setActionCommand("faster");
        b4 = new JButton("Slower");
        b4.setActionCommand("slower");
        b5 = new JButton("More Cap");
        b5.setActionCommand("morecap");
        b6 = new JButton("Quit");
        b6.setActionCommand("quit");
        b7 = new JButton("Less Cap");
        b7.setActionCommand("lesscap");
        

        // set us up as a listener to each button, so 
        // actionPerformed(...) gets called
        b0.addActionListener(this);
        b1.addActionListener(this);
        b2.addActionListener(this);
        b3.addActionListener(this);
        b4.addActionListener(this);
        b5.addActionListener(this);
        b6.addActionListener(this);
        b7.addActionListener(this);

        b0.setToolTipText("Start the simulation");
        b1.setToolTipText("Pause the animation");
        b2.setToolTipText("Continue the animation");
        b3.setToolTipText("Speed up the simulation");
        b4.setToolTipText("Slow down the simulation");
        b5.setToolTipText("Increase the capacity of channels");
        b6.setToolTipText("Quit the application");
        b7.setToolTipText("Decrease the capacity of channels");

        // add the buttons to us
        add(b0);
        add(b1);
        add(b2);
        add(b3);
        add(b4);
        add(b5);
        add(b7);
        add(b6);

    }

    /** This is called each time a button is pressed. 
     * Based on the button command the appropriate 
     * action is invoked in the DrawingArea. */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("pause")) {
            d.pause();
        } else if (e.getActionCommand().equals("continue")) { 
            d.unpause();
        } else if (e.getActionCommand().equals("start")) { 
            d.start();
        } else if (e.getActionCommand().equals("faster")) { 
            d.faster();
        } else if (e.getActionCommand().equals("slower")) { 
            d.slower();
        } else if (e.getActionCommand().equals("morecap")) { 
            d.moreCapacity();
        } else if (e.getActionCommand().equals("lesscap")) {
        	d.lessCapacity();
        } else if (e.getActionCommand().equals("quit")) { 
            d.quit();
            System.out.println("Quitting by button press.");
            System.exit(0);
        }
    }
}
 


/** The actual drawing area.  This also creates and communicates
 * with the actual Circuit simulation. */
class DrawingArea extends JPanel {

    /** timer for animation. */
    protected Timer timer;
    
    /** Actual circuit. */
    protected Circuit c;

    /** Builds the circuit and starts a timer to keep repainting. */
    public DrawingArea(Circuit c) {
        // create as double-buffered
        super(true);

        // Construct the circuit; this uses the loop template
        c.build();
        this.c = c;

        // for animations, use a Timer object.  This will
        // cause actionPerformed to be called every so
        // often (10ms here)
        timer = new Timer(10, new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    // repaint causes a paint event to be posted,
                    // which will cause paint to be called in due time
                    repaint();
                }    
            });
        // start the timer
        timer.start();
    }

    /** The paint method is called whenever the window needs to be 
     * repainted.  The timer started in the constructor will also 
     * cause this to be called repeatedly by posting paint events. */
    public void paintComponent(Graphics g) {
        // let superclass paint to fill in background
        super.paintComponent(g);
        
        // find out how big the window is 
        Insets insets = getInsets();

        // get the actual drawing area size 
        int currentWidth = getWidth() - insets.left - insets.right;
        int currentHeight = getHeight() - insets.top - insets.bottom;

        // draw the circuit
        c.draw(g);

        // draw the speed bar
        c.drawSpeed(g,currentWidth,currentHeight);
    }

    /** Stop the animation. */
    void pause() {
        c.pause();
    }

    /** Continue if paused. */
    void unpause() {
        c.resume();
    }

    /** Start the simulation. */
    void start() {
        c.start();
    }

    /** Speed up the simulation. */
    void faster() {
        c.faster();
    }

    /** Slow down the simulation. */
    void slower() {
        c.slower();
    }

    /** Request more channel capacity. */
    void moreCapacity() {
        c.moreCapacity();
    }
    
    /** Request less channel capacity. */
    void lessCapacity() {
    	c.lessCapacity();
    }

    /** Quit the simulation. */
    void quit() {
        c.stop();
    }
}
